package controler;

import java.io.File;
import java.time.LocalTime;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

import model.Alarm;
import model.AlarmList;
import model.FileType;
import model.Time;
import view.AlarmDialog;
import view.AlarmFrame;

public class Controler extends Thread {
	private AlarmFrame frame;
	private AlarmList alarms;
	public Controler() {
		frame = null;
		alarms = null;
  	}
	public void setView(AlarmFrame f) {
		frame = f;
	}
	public void setModel(AlarmList l) {
		alarms = l;
	}
	public void newAlarm() {
		try {
			Alarm alarm = new Alarm("nouvelle", "12:00");
			alarms.add(alarm);
		} catch(Exception e) { };
		alarms.sort();
		frame.updateAlarmList();
		frame.disableEditor();
		alarms.setChanged();
        frame.updateTitle();
	}
	public void loadAlarms() {
        JFileChooser fileChooser = new JFileChooser();
        FileType ft = FileType.BINARY;
        FileNameExtensionFilter filter = new FileNameExtensionFilter(ft.legend(), ft.extension());
        fileChooser.setFileFilter(filter);
        switch (fileChooser.showOpenDialog(frame)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
                if (alarms.load(fileChooser.getSelectedFile())) {
            		frame.updateAlarmList();
            		frame.disableEditor();
                    frame.updateTitle();
                    frame.allowSaveItem(true);
                } else
                    JOptionPane.showMessageDialog(frame,
                                                  "Un problme est survenu pendant la lecture du fichier.",
                                                  "Alarmes non charges",
                                                  JOptionPane.ERROR_MESSAGE);
                break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(frame,
                                              "Un problme est survenu lors du choix de fichier.",
                                              "Alarmes non charges",
                                              JOptionPane.ERROR_MESSAGE);
                break;
        }
	}
	public void saveAlarms() {
        if (!alarms.save()) {
            JOptionPane.showMessageDialog(frame,
                                          "Un problme est survenu pendant l'enregistrement.",
                                          "Alarmes non enregistres",
                                          JOptionPane.ERROR_MESSAGE);
        } else {
            frame.updateTitle();
        }
	}
	private File fileChoosedWithExtension(File file, FileNameExtensionFilter filter) {
		String[] extensions = filter.getExtensions();
		String fileName = file.getName().toLowerCase();
		if (fileName.endsWith('.' + extensions[0])) return file;
		file = new File(file.getAbsolutePath() + '.' + extensions[0]);
		return file;
	}
	public void saveAlarmsAs() {
        JFileChooser fileChooser = new JFileChooser();
        for(FileType ft : FileType.values()) {
            fileChooser.addChoosableFileFilter(new FileNameExtensionFilter(ft.legend(), ft.extension()));
        }
        fileChooser.setAcceptAllFileFilterUsed(false);
        switch (fileChooser.showSaveDialog(frame)) {
            case JFileChooser.CANCEL_OPTION  :
                break;
            case JFileChooser.APPROVE_OPTION :
                if (alarms.saveAs(fileChoosedWithExtension(fileChooser.getSelectedFile(), (FileNameExtensionFilter)fileChooser.getFileFilter()))) {
/*
                if (alarms.saveAs(fileChooser.getSelectedFile(),
                			      FileType.typeForExtension(((FileNameExtensionFilter)fileChooser.getFileFilter()).getExtensions()[0]))) {
*/
                	frame.updateTitle();
                } else {
                    JOptionPane.showMessageDialog(frame,
                                                  "Un problme est survenu pendant l'enregistrement des alarmes.",
                                                  "Alarmes non enregistres",
                                                  JOptionPane.ERROR_MESSAGE);
                }
                break;
            case JFileChooser.ERROR_OPTION   :
                JOptionPane.showMessageDialog(frame,
                                              "Un problme est survenu lors du choix de fichier pour l'enregistrement du dessin.\nLe dessin n'a pas t enregistr.",
                                              "Alarmes non enregistres",
                                              JOptionPane.ERROR_MESSAGE);
                break;
        }
	}
	public void changeAlarm(Alarm alarm, String name, String time) {
		alarm.setName(name);
		try { alarm.setTime(time); } catch(Exception e) { }
		alarms.sort();
		alarms.setChanged();
		frame.updateAlarmList();
		frame.disableEditor();
		frame.updateTitle();
	}
	public void removeAlarm(Alarm alarm) {
		alarms.remove(alarm);
		alarms.setChanged();
		frame.updateAlarmList();
		frame.disableEditor();
		frame.updateTitle();
	}
	public void run() {
		while (true) {
			LocalTime now = LocalTime.now();
			for (Alarm a : alarms) {
				Time t = a.getTime();
				if (t.hour() == now.getHour() && t.minute() == now.getMinute()) {
					(AlarmDialog.getInstance()).display(a);
					break;
				}
			}
			try { sleep(60000); } catch (InterruptedException e) { }
		}
	}
}
