package model;

import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.LinkedList;

@SuppressWarnings("serial")
public class AlarmList extends LinkedList<Alarm> implements Operable {
	private File file;
	private boolean changed;
	public AlarmList() {
		super();
		changed = false;
		file = null;
	}
	public boolean hasChanged() {
		return changed;
	}
	public void setChanged() {
		changed = true;
	}
	public boolean saveAs(File f) {
		file = f;
		String fileName = f.getAbsolutePath();
		int dotPos = fileName.lastIndexOf('.');
		String extension = fileName.substring(dotPos + 1);
		FileType ft = FileType.typeForExtension(extension);
		Save recorder = new Save(this);
		boolean saved = recorder.saveAs(fileName, ft);
		if (saved && ft == FileType.BINARY) changed = false;
		return saved;
	}
	public boolean save() {
		return saveAs(file);
	}
	public boolean load(File f) {
		file = f;
		try (DataInputStream dis = new DataInputStream(new FileInputStream(f))) {
			int n = dis.readInt();
			for(int i = 0; i < n; i++) {
				Alarm a = new Alarm();
				a.loadFrom(dis);
				add(a);
			}
		} catch (FileNotFoundException fe) {
			return false;
		} catch (IOException e) {
			return false;
		}
		changed = false;
		return true;
	}
    public String fileName() {
        String name = (changed) ? "*" : "";
        if (file == null) name += "Sans nom";
        else name += file.getName();
        return name; 
    }
	public void sort() {
/*
		sort(new Comparator<Alarm>() {
            public int compare(Alarm a1, Alarm a2) {
                return a1.compareTo(a2);
            }
        });
 */
		sort((Alarm a1, Alarm a2) -> a1.compareTo(a2));
	}
	public void performs(Operation op) throws Exception {
		op.doItOn(this);
	}
}
